/**************************************************
UZI (Unix Z80 Implementation) Kernel:  machdep.c
***************************************************/


#include "unix.h"
#include "extern.h"
#include <sys.h>

static char vector[3];  /* Place for interrupt vector */

int unix_extern(argn3, argn2, argn1, argn, uret, callno)
int16 argn3, argn2, argn1, argn;
char *uret;
int callno;
{
  return unix_extern(argn3,argn2,argn1,argn,uret,callno);
}

/* This is called at the very beginning to initialize everything. */
/* It is the equivalent of main() */

void main( int argc, char **argv ) {

  fs_init();

  /* dummy - compiler reported warnings */
  vector[0];
}

fs_init()
{
    di();
    stkreset();
    mz_init();
    panic( "Process management not covered yet.." );
    /* implemented to this point */
    
    /* Initialize the interrupt vector */
    initvec();
    inint = 0;
    udata.u_insys = 1;
    /* Turn off clock */
    mz_stop_clock();
    ei();

    init2();   /* in process.c */
}


/* This checks to see if a user-suppled address is legitimate */
valadr(base,size)
char *base;
uint16 size;
{
    if (base < PROGBASE || base+size >= (char *)&udata)
    {
        udata.u_error = EFAULT;
        return(0);
    }
    return(1);
}


/* This adds two tick counts together.
The t_time field holds up to one second of ticks,
while the t_date field counts minutes */

addtick(t1,t2)
time_t *t1, *t2;
{

    t1->t_time += t2->t_time;
    t1->t_date += t2->t_date;
    if (t1->t_time >= 60*TICKSPERSEC)
    {
        t1->t_time -= 60*TICKSPERSEC;
        ++t1->t_date;
    }
}

incrtick(t)
time_t *t;
{
    if (++t->t_time == 60*TICKSPERSEC)
    {
        t->t_time = 0;
        ++t->t_date;
    }
}


stkreset()
{
#asm
	ld	sp, ix
	pop	ix
	pop	iy
	pop	hl
        ld      sp, 0
        jp      (hl)
#endasm
}


tempstack()
{
#asm
	ld	sp, ix
	pop	ix
	pop	iy
	pop	hl
	ld	sp, 100h
	jp	(hl)
#endasm
}



initvec()
{
#asm
	ld	HL,_vector
        INC     HL
        ld      A,L
        AND     0FEH
        ld      L,A     ;set hl to first even address in vector[].
        ld      A,H
        ld      I,A     ;SET INTERRUPT REGISTER TO UPPER 8 BITS
        ld      A,L
        OUT     (076H),A    ;set external vector register with low order byte
        ld      DE,_service_i
        ld      (HL),E
        INC     HL
        ld      (HL),D     ;STORE ADDRESS OF SERVICE ROUTINE IN vector[].
#endasm
}



doexec()
{
#asm
	ld	l, (ix+6)
	ld	h, (ix+7)
	ld	sp, hl
	ld	a, 0c3h			; jump
	ld	(30h), a		; dest of RST6 instruction
	ld	hl, _unix_extern	; entry address
	ld	(31h), hl
	xor	a
	ld	(_udata + _OSYS), a
	jp	100h
#endasm
}


static int cursig;
static int (*curvec)();

/* This interrupt device routine calls the service routine of each device
that could have interrupted. */

service()
{
#asm
_service_i:
        PUSH    AF
        PUSH    BC
        PUSH    DE
        PUSH    HL
        PUSH    IX
        PUSH    IY
#endasm

    inint = 1;

    if (tty_int())
        goto found;
    if (clk_int())
        goto found;
/*  if (  ) ...   */

    warning("Spurious interrupt");

found:
    inint = 0;

    /* Deal with a pending caught signal, if any */
    if (!udata.u_insys)
        calltrap();
    ;

#asm
        POP     IY
        POP     IX
        POP     HL
        POP     DE
        POP     BC
        POP     AF
        EI
        RETI
#endasm
}



calltrap()
{
    /* Deal with a pending caught signal, if any. */
        /* udata.u_insys should be false, and interrupts enabled.
        remember, the user may never return from the trap routine */

    if (udata.u_cursig)
    {
        cursig = udata.u_cursig;
        curvec = udata.u_sigvec[cursig];
        udata.u_cursig = 0;
        udata.u_sigvec[cursig] = SIG_DFL;   /* Reset to default */
        ei();
        (*curvec)(cursig);
        di();
    } 
}



/* Port addresses of clock chip registers. */
sttime()
{
    panic("Calling sttime");
}


rdtime(tloc)
time_t *tloc;
{
    di();
    tloc->t_time = tod.t_time;
    tloc->t_date = tod.t_date;
    ei();
}


/* Disable interrupts */
di()
{
#asm
        di      ;disable interrupts
#endasm
}

/* Enable interrupts if we are not in service routine */
ei()
{
    if (inint)
        return;
    ;   /* Empty statement necessary to fool compiler */

#asm
        ei      ;disable interrupts
#endasm
}



/* This prints an error message and dies. */
panic(s)
char *s;
{
    di();
    inint = 1;
    kprintf("PANIC: %s\n",s);
    idump();
    abort();
}


warning(s)
char *s;
{
    kprintf("WARNING: %s\n",s);
}


puts(s)
char *s;
{
    while (*s)
        kputchar(*(s++));
}

kputchar(c)
int c;
{
    if (c == '\n')
        _putc('\r');
    _putc(c);
    if (c == '\t')
        puts("\177\177\177\177\177\177\177\177\177\177");
}



idump()
{
    inoptr ip;
    ptptr pp;
    extern struct cinode i_tab[];

    kprintf(
        "\tMAGIC\tDEV\tNUM\tMODE\tNLINK\t(DEV)\tREFS\tDIRTY err %d root %d\n",
            udata.u_error, root - i_tab);

    for (ip=i_tab; ip < i_tab+ITABSIZE; ++ip)
    {
        kprintf("%d\t%d\t%d\t%u\t0%o\t%d\t%d\t%d\t%d\n",
               ip-i_tab, ip->c_magic,ip->c_dev, ip->c_num,
               ip->c_node.i_mode,ip->c_node.i_nlink,ip->c_node.i_addr[0],
               ip->c_refs,ip->c_dirty);
/*****
        ifnot (ip->c_magic)     
            break;
******/
    }

    kprintf("\n\tSTAT\tWAIT\tPID\tPPTR\tALARM\tPENDING\tIGNORED\n");
    for (pp=ptab; pp < ptab+PTABSIZE; ++pp)
    {
        kprintf("%d\t%d\t0x%x\t%d\t%d\t%d\t0x%x\t0x%x\n",
               pp-ptab, pp->p_status, pp->p_wait,  pp->p_pid,
               pp->p_pptr-ptab, pp->p_alarm, pp->p_pending,
                pp->p_ignored);
        ifnot(pp->p_pptr)
            break;
    }   
    
    bufdump();

    kprintf("\ninsys %d ptab %d call %d cwd %d sp 0x%x\n",
        udata.u_insys,udata.u_ptab-ptab, udata.u_callno, udata.u_cwd-i_tab,
       udata.u_sp);
}



/* Short version of printf to save space */
kprintf(nargs)
        {
        register char **arg, *fmt;
        register c, base;
        char s[7];

        arg = (char **)&nargs + nargs;
        fmt = *arg;
        while (c = *fmt++) {
                if (c != '%') {
                        kputchar(c);
                        continue;
                        }
                switch (c = *fmt++) {
                case 'c':
                        kputchar(*--arg);
                        continue;
                case 'd':
                        base = -10;
                        goto prt;
                case 'o':
                        base = 8;
                        goto prt;
                case 'u':
                        base = 10;
                        goto prt;
                case 'x':
                        base = 16;
                prt:
                        puts(itoa((int)*--arg, s, base));
                        continue;
                case 's':
                        puts(*--arg);
                        continue;
                default:
                        kputchar(c);
                        continue;
                        }
                }
        }

